#version 330
 
// created by Thorsten Thormaehlen for educational purpose
   
in vec3 normalInterp;
in vec3 vertPos;

out vec4 outputColor;

uniform int mode;

struct LightSource {
  vec3 position;
  vec3 color;
  float constantAttenuation, linearAttenuation, quadraticAttenuation;
  float spotCutoff, spotExponent;
  vec3 spotDirection;
};

const int maxNumberOfLights = 3;
uniform int numberOfLights;
uniform LightSource lights[maxNumberOfLights];

const vec3 ambientColor = vec3(0.0, 0.0, 0.0);
const vec3 diffuseColor = vec3(0.5, 0.0, 0.0);
const vec3 specColor = vec3(1.0, 1.0, 1.0);

void main() {

    vec3 normal = normalize(normalInterp);
    vec3 contrib = ambientColor;

    for(int i=0; i < maxNumberOfLights; i++) {

      vec3 lightVec = (lights[i].position - vertPos);
      float d = length(lightVec); // distance to light

      vec3 lightDir = normalize(lightVec);

      // attenuation due to spot
      float attenuation = dot(-lightDir,lights[i].spotDirection);
      if(attenuation < lights[i].spotCutoff) {
         attenuation = 0.0;
      }else{
         attenuation = pow(attenuation, lights[i].spotExponent);
      }

      // atttenuation due to distance fall off
      attenuation *= 1.0 / (lights[i].constantAttenuation +
                            lights[i].linearAttenuation * d +
                            lights[i].quadraticAttenuation * d * d);

      float lambertian = max(dot(lightDir,normal), 0.0);
      float specular = 0.0;

      if(lambertian > 0.0) {

          vec3 viewDir = normalize(-vertPos);

          // this is blinn phong
          vec3 halfDir = normalize(lightDir + viewDir);
          float specAngle = max(dot(halfDir, normal), 0.0);
          specular = pow(specAngle, 16.0);
       }
       vec3 lightContrib = lambertian * diffuseColor;
       lightContrib += specular * specColor;
       lightContrib *= lights[i].color;
       lightContrib *= attenuation;

       contrib += lightContrib;
    }

    outputColor = vec4(contrib, 1.0);
}
